// RockTexture.h
/* RockTexture.h declares class RockTexture.
 * 
 *    The value of myRockTextureName is one of the following:
 *       COARSE, INTERMEDIATE, FINE, 
 ***************************************************************/

#ifndef ROCK_TEXTURE
#define ROCK_TEXTURE

enum RockTextureName {TEXTURE_UNDERFLOW = -1,
                      COARSE, INTERMEDIATE, FINE,
                      NUMBER_OF_TEXTURES,
                      TEXTURE_OVERFLOW = NUMBER_OF_TEXTURES};

#include <iostream.h>                     // istream, ostream

class RockTexture
{
 public:                                  // constructors:
   RockTexture();                            //  default value
   RockTexture(RockTextureName initialRock); //  explicit value

   void Read(istream & in);               // input
   void Print(ostream & out) const;       // output

   RockTexture operator++();              // prefix ++
   RockTexture operator++(int);           // postfix ++
   RockTexture operator--();              // prefix --
   RockTexture operator--(int);           // postfix --

   friend bool operator==(const RockTexture & left, const RockTexture & right);
   friend bool operator!=(const RockTexture & left, const RockTexture & right);
   friend bool operator<(const RockTexture & left, const RockTexture & right);
   friend bool operator>(const RockTexture & left, const RockTexture & right);
   friend bool operator<=(const RockTexture & left, const RockTexture & right);
   friend bool operator>=(const RockTexture & left, const RockTexture & right);

 private:
   RockTextureName myRockTextureName;
};

// -------- Non-member input ---------------------------------------
inline istream & operator>>(istream & in, RockTexture & theRockTexture)
{
   theRockTexture.Read(in);
   return in;
}

// -------- Non-member output --------------------------------------
inline ostream & operator<<(ostream & out, const RockTexture & theRockTexture)
{
   theRockTexture.Print(out);
   return out;
}

// -------- Initialize me (default-value) --------------------------
inline RockTexture::RockTexture()
{
   myRockTextureName = COARSE;
}

// -------- Initialize me (explicit-value) -------------------------
#include <cassert>
inline RockTexture::RockTexture(RockTextureName initialRockTextureName)
{
   assert(COARSE <= initialRockTextureName && 
          initialRockTextureName < NUMBER_OF_TEXTURES);
   myRockTextureName = initialRockTextureName;
}

// --------- Compare me and anotherRockTexture using == ---------------
inline bool operator==(const RockTexture & left, const RockTexture & right)
{
   return left.myRockTextureName == right.myRockTextureName;
}

// ---  Inline definitions for operators !=, <, >, <=, and >= are
// ---  essentially the same and are omitted here to save space.

#endif
